/*
 * Copyright (c) 2010-2022 Belledonne Communications SARL.
 *
 * This file is part of Liblinphone 
 * (see https://gitlab.linphone.org/BC/public/liblinphone).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MSC_VER
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif // _MSC_VER

#ifndef _EKT_INFO_HH
#define _EKT_INFO_HH

#include <cstdint>
#include <string>
#include "object.hh"




struct _LinphoneEktInfo;


namespace linphone {

	class Dictionary;
	class Buffer;
	class Address;

	/**
	 * @brief Object representing all informations present in an Encrypted Key
	 * Transport event. 
	 *
	 */
	class EktInfo: public Object {
	
		public:

			LINPHONECXX_PUBLIC EktInfo(void *ptr, bool takeRef=true);
			LINPHONECXX_PUBLIC _LinphoneEktInfo *cPtr() {return (_LinphoneEktInfo *)mPrivPtr;}

	
		public:
			
		
			/**
			 * @brief Get ciphers. 
			 *
			 * @return a @ref Dictionary object.    
			 */
			LINPHONECXX_PUBLIC std::shared_ptr<linphone::Dictionary> getCiphers() const;
			
			/**
			 * @brief Set ciphers. 
			 *
			 * @param ciphers the ciphers to set.    
			 */
			LINPHONECXX_PUBLIC void setCiphers(const std::shared_ptr<linphone::Dictionary> & ciphers);
			
			/**
			 * @brief Get cSPI. 
			 *
			 * @return a @ref Address object.       
			 */
			LINPHONECXX_PUBLIC std::shared_ptr<linphone::Buffer> getCspi() const;
			
			/**
			 * @brief Set cSPI. 
			 *
			 * @param cspi the address to set.    
			 */
			LINPHONECXX_PUBLIC void setCspi(const std::shared_ptr<const linphone::Buffer> & cspi);
			
			/**
			 * @brief Get from address. 
			 *
			 * @return a @ref Address object.    
			 */
			LINPHONECXX_PUBLIC std::shared_ptr<const linphone::Address> getFromAddress() const;
			
			/**
			 * @brief Set from address. 
			 *
			 * @param from the address to set.    
			 */
			LINPHONECXX_PUBLIC void setFromAddress(const std::shared_ptr<const linphone::Address> & from);
			
			/**
			 * @brief Get sSPI. 
			 *
			 * @return a 16-bits unsigned int representing the sSPI. 
			 */
			LINPHONECXX_PUBLIC uint16_t getSspi() const;
			
			/**
			 * @brief Set sSPI. 
			 *
			 * @param sspi the sspi to set. 
			 */
			LINPHONECXX_PUBLIC void setSspi(uint16_t sspi);
			
			/**
			 * @brief Add a cipher to the chipher map. 
			 *
			 * @param to the address to set.    
			 * @param cipher the cipher to set.    
			 */
			LINPHONECXX_PUBLIC void addCipher(const std::string & to, const std::shared_ptr<const linphone::Buffer> & cipher);
			
			
	};

};

#endif // _EKT_INFO_HH

#ifndef _MSC_VER
#pragma GCC diagnostic pop
#endif // _MSC_VER
