/*
 * Copyright (c) 2010-2022 Belledonne Communications SARL.
 *
 * This file is part of Liblinphone 
 * (see https://gitlab.linphone.org/BC/public/liblinphone).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MSC_VER
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif // _MSC_VER

#ifndef _CARD_DAV_PARAMS_HH
#define _CARD_DAV_PARAMS_HH

#include <list>
#include <string>
#include "object.hh"




struct _LinphoneCardDavParams;


namespace linphone {

	class CardDavParams;

	/**
	 * @brief The @ref CardDavParams object represents a remote CardDAV server used by
	 * @ref MagicSearch as a plugin source. 
	 *
	 */
	class CardDavParams: public Object {
	
		public:

			LINPHONECXX_PUBLIC CardDavParams(void *ptr, bool takeRef=true);
			LINPHONECXX_PUBLIC _LinphoneCardDavParams *cPtr() {return (_LinphoneCardDavParams *)mPrivPtr;}

	
		public:
			
		
			/**
			 * @brief Gets the list of vCard RFC fields to use to match the domain filter on. 
			 *
			 * For example you can use "IMPP". 
			 * @return The list of vCard fields to make the query on using domain filter.     
			 *    
			 */
			LINPHONECXX_PUBLIC std::list<std::string> getDomainFields() const;
			
			/**
			 * @brief Sets the list of vCard RFC fields to use to match the domain filter on. 
			 *
			 * For example you can use "IMPP". 
			 * @param list the list of vCard RFC fields to use to match the domain filter on. 
			 *     
			 */
			LINPHONECXX_PUBLIC void setDomainFields(const std::list<std::string> & list);
			
			/**
			 * @brief Gets the matching policy: true if the remote vCard value must match the
			 * filter exactly, false if it only needs to contain the filter. 
			 *
			 * @return The matching policy: true if the remote vCard value must match the
			 * filter exactly, false if it only needs to contain the filter. 
			 */
			LINPHONECXX_PUBLIC bool getUseExactMatchPolicy() const;
			
			/**
			 * @brief Sets the matching policy: true if the remote vCard value must match the
			 * filter exactly, false if it only needs to contain the filter. 
			 *
			 * @param exactMatch the policy to use: true if the remote vCard value must match
			 * the filter exactly, false if it only needs to contain the filter. 
			 */
			LINPHONECXX_PUBLIC void setUseExactMatchPolicy(bool exactMatch);
			
			/**
			 * @brief Gets the list of vCard RFC fields to use to match user input text on. 
			 *
			 * For example you can use "FN", "N", "IMPP", "ORG", etc... 
			 * @return The list of vCard fields to make the query on using user input.         
			 */
			LINPHONECXX_PUBLIC std::list<std::string> getUserInputFields() const;
			
			/**
			 * @brief Sets the list of vCard RFC fields to use to match user input text on. 
			 *
			 * For example you can use "FN", "N", "IMPP", "ORG", etc... 
			 * @param list the list of vCard RFC fields to use to match user input text on.   
			 *   
			 */
			LINPHONECXX_PUBLIC void setUserInputFields(const std::list<std::string> & list);
			
			/**
			 * @brief Instantiates a new @ref CardDavParams with values from source. 
			 *
			 * @return The newly created @ref CardDavParams object.    
			 */
			LINPHONECXX_PUBLIC std::shared_ptr<linphone::CardDavParams> clone() const;
			
			
	};

};

#endif // _CARD_DAV_PARAMS_HH

#ifndef _MSC_VER
#pragma GCC diagnostic pop
#endif // _MSC_VER
