/*
 * Copyright (c) 2010-2022 Belledonne Communications SARL.
 *
 * This file is part of Liblinphone 
 * (see https://gitlab.linphone.org/BC/public/liblinphone).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MSC_VER
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif // _MSC_VER

#ifndef _FRIEND_DEVICE_HH
#define _FRIEND_DEVICE_HH

#include <string>
#include "enums.hh"
#include "object.hh"




struct _LinphoneFriendDevice;


namespace linphone {

	class Address;
	class FriendDevice;

	/**
	 * @brief Object that represents a @ref Friend's device (name, trust level) for a
	 * given SIP address. 
	 *
	 */
	class FriendDevice: public Object {
	
		public:

			LINPHONECXX_PUBLIC FriendDevice(void *ptr, bool takeRef=true);
			LINPHONECXX_PUBLIC _LinphoneFriendDevice *cPtr() {return (_LinphoneFriendDevice *)mPrivPtr;}

	
		public:
			
		
			/**
			 * @brief Gets the address associated to this device. 
			 *
			 * @return the address (including gruu) to which this device is linked.    
			 */
			LINPHONECXX_PUBLIC std::shared_ptr<const linphone::Address> getAddress() const;
			
			/**
			 * @brief Gets the display name of this device. 
			 *
			 * @return the name of the device.    
			 */
			LINPHONECXX_PUBLIC std::string getDisplayName() const;
			
			/**
			 * @brief Gets the current security level of this device. 
			 *
			 * @return the current @ref SecurityLevel of the device. 
			 */
			LINPHONECXX_PUBLIC linphone::SecurityLevel getSecurityLevel() const;
			
			/**
			 * @brief Clones a device. 
			 *
			 * @return The newly created @ref FriendDevice object.    
			 */
			LINPHONECXX_PUBLIC std::shared_ptr<linphone::FriendDevice> clone() const;
			
			
	};

};

#endif // _FRIEND_DEVICE_HH

#ifndef _MSC_VER
#pragma GCC diagnostic pop
#endif // _MSC_VER
