/*
 * Copyright (c) 2010-2022 Belledonne Communications SARL.
 *
 * This file is part of Liblinphone 
 * (see https://gitlab.linphone.org/BC/public/liblinphone).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MSC_VER
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif // _MSC_VER

#ifndef _ACCOUNT_MANAGER_SERVICES_REQUEST_LISTENER_HH
#define _ACCOUNT_MANAGER_SERVICES_REQUEST_LISTENER_HH

#include <string>
#include <list>
#include "object.hh"






namespace linphone {

	class AccountManagerServicesRequest;
	class Dictionary;
	class AccountDevice;

	/**
	 * @brief An object to handle the callbacks for @ref AccountManagerServicesRequest
	 * object. 
	 *
	 */
	class AccountManagerServicesRequestListener: public Listener {
	
		public:


	
		public:
			
		
			/**
			 * @brief Callback for notifying a request was successful. 
			 *
			 * @param request @ref AccountManagerServicesRequest object.    
			 * @param data any relevant data depending on the request (for example SIP
			 * identity for account creation).    
			 */
			LINPHONECXX_PUBLIC virtual void onRequestSuccessful(const std::shared_ptr<const linphone::AccountManagerServicesRequest> & request, const std::string & data) {};
			
			/**
			 * @brief Callback for notifying a request has failed. 
			 *
			 * @param request @ref AccountManagerServicesRequest object.    
			 * @param statusCode The error status code. 
			 * @param errorMessage An error message, if any.    
			 * @param parameterErrors A @ref Dictionary with parameter specific errors, if
			 * any.    
			 */
			LINPHONECXX_PUBLIC virtual void onRequestError(const std::shared_ptr<const linphone::AccountManagerServicesRequest> & request, int statusCode, const std::string & errorMessage, const std::shared_ptr<const linphone::Dictionary> & parameterErrors) {};
			
			/**
			 * @brief Callback for notifying when the
			 * #LinphoneAccountManagerServicesRequestGetDevicesList request has results
			 * available. 
			 *
			 * @param request @ref AccountManagerServicesRequest object.    
			 * @param devicesList the   of fetched devices.    
			 */
			LINPHONECXX_PUBLIC virtual void onDevicesListFetched(const std::shared_ptr<const linphone::AccountManagerServicesRequest> & request, const std::list<std::shared_ptr<linphone::AccountDevice>> & devicesList) {};
			
			
	};

};

#endif // _ACCOUNT_MANAGER_SERVICES_REQUEST_LISTENER_HH

#ifndef _MSC_VER
#pragma GCC diagnostic pop
#endif // _MSC_VER
